#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w
use strict;
use Mail::Address;
use PureMessage::Manager::Lang;
use PureMessage::Enduser::Request qw(%in);
use PureMessage::Enduser::Response;
use PureMessage::Enduser::UI::User;

use ActiveState::I18N::UTF8 qw(utf8_header utf8_length);
use PureMessage::UTF8::Config qw(%utf8_config);

my $REQUEST = new PureMessage::Enduser::Request;
my $r = $REQUEST->get_response('', stderr_to_stdout => 0);

# set language name space
PureMessage::Manager::Lang::set_namespace("EUUI/messages");

# message types must be either 'blocked' or 'deleted'
#   - defaults to 'blocked'

my $message_type = $in{'message_type'};
if ((! defined $message_type) || (($message_type ne 'blocked') && ($message_type ne 'deleted'))) {
    $message_type = 'blocked';
}

my ($sort_from, $sort_date, $sort_probability, $sort_subject);
my $content_sort_by = $in{'content_sort_by'};
if ((defined $content_sort_by) && (($content_sort_by eq 'from') || ($content_sort_by eq 'h_from:addr'))) {
    $content_sort_by = 'h_from:addr';
    $sort_from = 1;
} elsif ((defined $content_sort_by) && (($content_sort_by eq 'date') || ($content_sort_by eq 'm_date'))) {
    $content_sort_by = 'm_date';
    $sort_date = 1;
} elsif ((defined $content_sort_by) && (($content_sort_by eq 'probability') || ($content_sort_by eq 'm_spam_probability'))) {
    $content_sort_by = 'm_spam_probability';
    $sort_probability = 1;
} elsif ((defined $content_sort_by) && (($content_sort_by eq 'subject') || ($content_sort_by eq 'c_subject_utf8'))) {
    $content_sort_by = 'c_subject_utf8';
    $sort_subject = 1;
} else {
    $content_sort_by = 'm_spam_probability';
    $sort_probability = 1;
}

# if there is not field to sort by, the order is irrelevant   
my ($order_ascending, $order_descending);
my $sort_order = $in{'sort_order'};
my $cgi_sort_order;
if (defined $content_sort_by) {
    if ((defined $sort_order) && ($sort_order eq 'ascending')) {
	$sort_order = 0;
	$cgi_sort_order = 'ascending';
	$order_ascending = 1;
	$order_descending = 0;
    } elsif ((defined $sort_order) && ($sort_order eq 'descending')) {
	$sort_order = 1;
	$cgi_sort_order = 'descending';
	$order_ascending = 0;
	$order_descending = 1;
    } else {
	$sort_order = 0;
	$cgi_sort_order = 'ascending';
	$order_ascending = 1;
	$order_descending = 0;
    }
}

my ($title, $view, $buttons_blocked, $buttons_deleted);
if ($message_type eq 'blocked') {
    $title = 'title_blocked_messages';
    $view = 'visible';
    $buttons_blocked = 1;
    $buttons_deleted = 0;
} else {
    $title = 'title_deleted_messages';
    $view = 'delete';
    $buttons_blocked = 0;
    $buttons_deleted = 1;
}

$r->set_title($title);
$r->{debug} =0;

# user object does data retrieval
my $user;
if (! $r->authorize_user) {
    $r->send();
    exit;
} else {
    $user = $r->user();
}

# set the language
my $lang = $user->language();
$r->errors($user, $0, 'language', 'get_language_error');

my @messids;
if ((ref($in{'selectsingle'})) && (ref($in{'selectsingle'}) eq 'ARRAY')) {
    @messids = @{$in{'selectsingle'}};
} else {
    push (@messids, $in{'selectsingle'}) if defined $in{'selectsingle'};
}

if ((defined $in{'accept'}) && ($in{'accept'} eq lang_text_utf8($lang, 'deliver_message_button'))) {
    if (! @messids) {
        $r->message(lang_text_utf8($lang, 'no_messages_selected'));
    } else {
	my $init_messids = scalar(@messids);
	my $tries = 1;
	# try 10 times
	while ((scalar(@messids)) && ($tries <= 10)) {
	    my @new_messids;
	    # bug - returns (1 => undef) every time ...
	    my %return_ids = $user->approve_msg(@messids);
	    $r->errors($user, $0, 'approve_msg', 'excessive_list_items');
	    foreach my $ret_id (keys %return_ids) {
		if (!$return_ids{$ret_id}) {
		    push(@new_messids, $ret_id);
		}
	    }
	    $tries++;
	    @messids = @new_messids;
	}
	my $final_messids = scalar(@messids);
	if (($final_messids > 0) && ($final_messids == $init_messids)) {
	    # no messages delivered
	    $r->raise_error(lang_text_utf8($lang, 'could_not_deliver'));
	} elsif (($final_messids > 0) && ($final_messids < $init_messids)) {
	    # some messages delivered
	    $r->raise_error(lang_text_utf8($lang, 'could_deliver_some'));
	} elsif ($final_messids == 0) {
	    $r->message(lang_text_utf8($lang, 'deliver_success'));
	} else {
	    $r->raise_error(lang_text_utf8($lang, 'unknown_deliver_error'));
	}
    }
}

if ((defined $in{'delete'}) && ($in{'delete'} eq lang_text_utf8($lang, 'delete_message_button'))) {
    if (! @messids) {
        $r->message(lang_text_utf8($lang, 'no_messages_selected'));
    } else {
	my $init_messids = scalar(@messids);
	my $tries = 1;
	# try 10 times
	while ((scalar(@messids)) && ($tries <= 10)) {
	    my @new_messids;
	    # bug - returns (1 => undef) every time ...
	    my %return_ids = $user->delete_msg(@messids);
	    $r->errors($user, $0, 'delete_msg', 'excessive_list_items');
	    foreach my $ret_id (keys %return_ids) {
		if (!$return_ids{$ret_id}) {
		    push(@new_messids, $ret_id);
		}
	    }
	    $tries++;
	    @messids = @new_messids;
	}
	my $final_messids = scalar(@messids);
	if (($final_messids > 0) && ($final_messids == $init_messids)) {
	    # no messages delivered
	    $r->raise_error(lang_text_utf8($lang, 'could_not_delete'));
	} elsif (($final_messids > 0) && ($final_messids < $init_messids)) {
	    # some messages delivered
	    $r->raise_error(lang_text_utf8($lang, 'could_delete_some'));
	} elsif ($final_messids == 0) {
	    $r->message(lang_text_utf8($lang, 'delete_success'));
	} else {
	    $r->raise_error(lang_text_utf8($lang, 'unknown_delete_error'));
	}
    }
}

if ((defined $in{'acceptapprove'}) && ($in{'acceptapprove'} eq lang_text_utf8($lang, 'acceptapprove_message_button'))) {
    if (! @messids) {
        $r->message(lang_text_utf8($lang, 'no_messages_selected'));
    } else {
        my @id_meta;
        foreach my $id (@messids) {
            my $meta = {'mess_id' => $id};
            push (@id_meta, $meta);
        }
        my @message_meta = $user->get_msg_meta(@messids);
        $r->errors($user, $0, 'get_msg_meta', 'excessive_list_items');
        if (! @message_meta) {
            $r->raise_error(lang_text_utf8($lang, 'no_message_summary'));
        } else {
            my %features  = $user->get_features();
            $r->errors($user, $0, 'get_features');
            my $list_wildcard = $features{'list.wildcard'};

            my @address_meta;
            foreach my $msg_meta (@message_meta) {
                my ($from_address) = Mail::Address->parse($msg_meta->{'h_from'});
                if (! $from_address) {
                    $r->raise_error(lang_text_utf8($lang, 'invalid_address') . $msg_meta->{'h_from'});
                    next;
                }
                my $domain_address = $from_address->host();
                $from_address = $from_address->address();
                my $add_meta = {'approve_email_address' => $from_address,
                                'domain_approve_email_address' =>$domain_address,
				'action_approve' => lang_text_utf8($lang, 'action_approve'),
				'action_approve_domain' => lang_text_utf8($lang, 'action_approve_domain'),
				'feature_wildcards' => $list_wildcard,
                            };
                push (@address_meta, $add_meta);
            }
            my $page = ((defined $in{'page'}) && ($in{'page'} > 0)) ? $in{'page'} : 1;
            my $content_tmpl = $r->new_tmpl('content_add_approve.html');
            $content_tmpl->param('button_approve' => lang_text_utf8($lang, 'button_approve'),
				 'button_cancel' => lang_text_utf8($lang, 'button_cancel'),
				 'approve_sender' => lang_text_utf8($lang, 'approve_sender'),
				 'message_ids' => \@id_meta,
                                 'add_senders' => \@address_meta,
                                 'content_message_hidden_page' => $page,
				 );
            $r->add_part($content_tmpl);
            $r->send;
            exit;
        }
    }
}

if ((defined $in{'save'}) && ($in{'save'} eq lang_text_utf8($lang, 'button_approve'))) {
    my @messids;
    if ((ref($in{'mess_id'})) && (ref($in{'mess_id'}) eq 'ARRAY')) {
	@messids = @{$in{'mess_id'}};
    } else {
	push (@messids, $in{'mess_id'}) if defined $in{'mess_id'};
    }

    my $init_messids = scalar(@messids);
    my $tries = 1;
    # try 10 times
    while ((scalar(@messids)) && ($tries <= 10)) {
	my @new_messids;
	# bug - returns (1 => undef) every time ...
	my %return_ids = $user->approve_msg(@messids);
	$r->errors($user, $0, 'approve_msg', 'excessive_list_items');
	foreach my $ret_id (keys %return_ids) {
	    if (!$return_ids{$ret_id}) {
		push(@new_messids, $ret_id);
	    }
	}
	$tries++;
	@messids = @new_messids;
    }
    my $final_messids = scalar(@messids);
    if (($final_messids > 0) && ($final_messids == $init_messids)) {
	# no messages delivered
	$r->raise_error(lang_text_utf8($lang, 'could_not_deliver'));
    } elsif (($final_messids > 0) && ($final_messids < $init_messids)) {
	# some messages delivered
	$r->raise_error(lang_text_utf8($lang, 'could_deliver_some'));
    } elsif ($final_messids == 0) {
	$r->message(lang_text_utf8($lang, 'deliver_success'));
    } else {
	$r->raise_error(lang_text_utf8($lang, 'unknown_deliver_error'));
    }

    my $key_val;
    my @app_adds;
    foreach my $parm (keys %in) {
	# look at only the correct parameter names
	if ($parm =~ /^approve_address_/) {
            if (ref($in{$parm}) eq "ARRAY") {                                   
                $key_val = $in{$parm}->[0];                                     
            } else {                                                            
                $key_val = $in{$parm};                                          
            }

	    if ($key_val =~ /\@/) {
		push(@app_adds, $key_val);
	    } else {
		push(@app_adds, '@' . $key_val);
	    }
	}
    }

    # even if $senders is undef that's OK, because it means the white list is empty
    my $senders = $user->get_items('whitelist');
    $r->errors($user, $0, 'get_items: whitelist', 'excessive_list_items');
    my %seen_senders;
    my @unique_senders;
    foreach my $sender (@$senders) {
	unless ($seen_senders{$sender}) {
	    $seen_senders{$sender} = 1;
	    push(@unique_senders, $sender);
	}
    }
    $senders = \@unique_senders;
    foreach my $app_add (@app_adds) {
	unless ($seen_senders{$app_add}) {
	    $seen_senders{$app_add} = 1;
	    push (@$senders, $app_add);
	}
    }
    my $ret_value = $user->set_items('whitelist', $senders);
    $r->errors($user, $0, 'set_items: whitelist', 'excessive_list_items');
    unless ($ret_value) {
	$r->raise_error(lang_text_utf8($lang, 'could_not_save'));
    }
    $r->message(lang_text_utf8($lang, 'deliver_success'));
}

if ((defined $in{'undelete'}) && ($in{'undelete'} eq lang_text_utf8($lang, 'undelete_message_button'))) {
    if (! @messids) {
        $r->message(lang_text_utf8($lang, 'no_messages_selected'));
    } else {
	my $init_messids = scalar(@messids);
	my $tries = 1;
	# try 10 times
	while ((scalar(@messids)) && ($tries <= 10)) {
	    my @new_messids;
	    # bug - returns (1 => undef) every time ...
	    my %return_ids = $user->undelete_msg(@messids);
	    $r->errors($user, $0, 'undelete_msg', 'excessive_list_items');
	    foreach my $ret_id (keys %return_ids) {
		if (!$return_ids{$ret_id}) {
		    push(@new_messids, $ret_id);
		}
	    }
	    $tries++;
	    @messids = @new_messids;
	}
	my $final_messids = scalar(@messids);
	if (($final_messids > 0) && ($final_messids == $init_messids)) {
	    # no messages delivered
	    $r->raise_error(lang_text_utf8($lang, 'could_not_restore'));
	} elsif (($final_messids > 0) && ($final_messids < $init_messids)) {
	    # some messages delivered
	    $r->raise_error(lang_text_utf8($lang, 'could_restore_some'));
	} elsif ($final_messids == 0) {
	    $r->message(lang_text_utf8($lang, 'restore_success'));
	} else {
	    $r->raise_error(lang_text_utf8($lang, 'unknown_restore_error'));
	}
    }
}

if ((defined $in{'deleteall'}) && ($in{'deleteall'} eq lang_text_utf8($lang, 'deleteall_message_button'))) {
    my $delete_tmpl = $r->new_tmpl('confirm_delete_all.html');
    $delete_tmpl->param('all_delete_messages' => lang_text_utf8($lang, 'all_delete_messages'),
			'delete_all_confirm' => lang_text_utf8($lang, 'delete_all_confirm'),
			'ok' => lang_text_utf8($lang, 'ok'),
			'cancel' => lang_text_utf8($lang, 'cancel'),
			);
    $r->add_part($delete_tmpl);
    $r->send;
    exit;
}

if ((defined $in{'confirm_delete_all'}) && ($in{'confirm_delete_all'} eq lang_text_utf8($lang, 'ok'))) {
    my $mess_count = $user->get_qview($view);
    $r->errors($user, $0, 'get_qview', 'excessive_list_items');
    my @qrows = $user->get_qrows($view, 0, ($mess_count - 1));
    $r->errors($user, $0, 'get_qview', 'excessive_list_items');
    my @messids;
    foreach my $mess (@qrows) {
	push(@messids, $mess->{'qid'});
    }
    my $init_messids = scalar(@messids);
    my $tries = 1;
    # try 10 times
    while ((scalar(@messids)) && ($tries <= 10)) {
	my @new_messids;
	# bug - returns (1 => undef) every time ...
	my %return_ids = $user->delete_msg(@messids);
	$r->errors($user, $0, 'delete_msg', 'excessive_list_items');
	foreach my $ret_id (keys %return_ids) {
	    if (!$return_ids{$ret_id}) {
		push(@new_messids, $ret_id);
	    }
	}
	$tries++;
	@messids = @new_messids;
    }
    my $final_messids = scalar(@messids);
    if (($final_messids > 0) && ($final_messids == $init_messids)) {
	# no messages delivered
	$r->raise_error(lang_text_utf8($lang, 'could_not_delete'));
    } elsif (($final_messids > 0) && ($final_messids < $init_messids)) {
	# some messages delivered
	$r->raise_error(lang_text_utf8($lang, 'could_delete_some'));
    } elsif ($final_messids == 0) {
	$r->message(lang_text_utf8($lang, 'delete_success'));
    } else {
	$r->raise_error(lang_text_utf8($lang, 'unknown_delete_error'));
    }
}

if (defined $in{'not_spam'} && ($in{'not_spam'} eq lang_text_utf8($lang, 'not_spam'))) {
    if (! @messids) {
        $r->message(lang_text_utf8($lang, 'no_messages_selected'));
    } else {
        if($user->not_spam_msg(@messids)) {
            $r->message(lang_text_utf8($lang, 'forward_success'));
        } else {
            $r->raise_error(lang_text_utf8($lang, 'unknown_deliver_error'));
        }
    }
}

my $mess_count = $user->get_qview($view);
$r->errors($user, $0, 'get_qview', 'excessive_list_items');

if (! $mess_count) {
    if ($message_type eq 'blocked') {
	$r->add_part("<p>" . lang_text_utf8($lang, 'no_blocked_messages') . "</p>\n");
    }
    if ($message_type eq 'deleted') {
	$r->add_part("<p>" . lang_text_utf8($lang, 'no_deleted_messages') . "</p>\n");
    }
    $r->send;
    exit;
}

my $pages = int($mess_count / 20);
if (($mess_count % 20) != 0) {
    $pages++;
}

my $page = ((defined $in{'page'}) && ($in{'page'} > 0)) ? $in{'page'} : 1;
$page = ($page > $pages) ? $pages : $page;

if (((defined $in{'next'}) && ($in{'next'} ne '')) && ($page < $pages)) {
    $page++;
}
if (((defined $in{'previous'}) && ($in{'previous'} ne '')) && ($page > 1)) {
    $page--;
}

if ((defined $in{'top_page'}) && ($in{'top_page'} =~ /\d/)) {
    $page = $in{'top_page'};
}

if ((defined $in{'bottom_page'}) && ($in{'bottom_page'} =~ /\d/)) {
    $page = $in{'bottom_page'};
}

my $previous = ($page > 1) ? 1 : 0;
my $next = ($page < $pages) ? 1 : 0;

my @pgs;
for (my $i = 1; $i <= $pages; $i++) {
    next if ($i == $page);
    push(@pgs, {'page_label' => lang_text_utf8($lang, 'page_label'),
		'content_message_page_number' => $i});
}

my $start = (($page - 1) * 20);
my $end = $start + 19;
$end = ($end > ($mess_count - 1)) ? ($mess_count - 1) : $end;

my @qrows = $user->get_qrows($view, $start, $end, $content_sort_by, $sort_order);
$r->errors($user, $0, 'get_qrows', 'excessive_list_items');

my %features = $user->get_features();
$r->errors($user, $0, 'get_features', 'excessive_list_items');
my $quarantine_body;
if (! %features) {
    $quarantine_body = 1;
} else {
    $quarantine_body = $features{'quarantine.body'};
}

if (! $features{'quarantine.messages'}) {
    $r->raise_error(lang_text_utf8($lang, 'feature_disabled'));
    $r->send;
    exit;
}

my @messages;
foreach my $mess (@qrows) {
    my ($from) = Mail::Address->parse($mess->{'h_from'});
    if ((! defined $from) || (! $from)) {
	$from = lang_text_utf8($lang, 'no_from');
    } else {
	$from = $from->address();
    }
    utf8_header($from, \%utf8_config);
    if (length($from) > 40) {
	$from = substr($from, 0, utf8_length($from, 40)) . '...';
    }
    my $subject = $mess->{'c_subject_utf8'};
    if ((! defined $subject) || (! $subject)) {
	$subject = lang_text_utf8($lang, 'no_subject');
    } elsif (length($subject) > 40) {
	$subject = substr($subject, 0, utf8_length($subject, 40)) . '...';
    }
    my $date = $mess->{'m_date:tzt'};
    my $prob = sprintf("%3.1f%%", (100 * $mess->{'m_spam_probability'}));
    my $clean_qid = $mess->{'qid'};
    $clean_qid =~ s/\W//g;
    my $mess_ref = {'content_message_list_value' => $mess->{'qid'},
		    'clean_qid' => $clean_qid,
		    'content_message_list_from' => $mess->{'h_from'},
		    'content_message_list_from_short' => $from,
		    'content_message_list_subject' => $mess->{'c_subject_utf8'},
		    'content_message_list_subject_short' => $subject,
		    'quarantine_body' => $quarantine_body,
		    'content_message_list_date' => $date,
		    'content_message_list_probability' => $prob,
		};
    push(@messages, $mess_ref);
}

my $content_tmpl = $r->new_tmpl('content_message.html');
$content_tmpl->param('deliver_message_button' => lang_text_utf8($lang, 'deliver_message_button'),
		     'delete_message_button' => lang_text_utf8($lang, 'delete_message_button'),
		     'acceptapprove_message_button' => lang_text_utf8($lang, 'acceptapprove_message_button'),
		     'deleteall_message_button' => lang_text_utf8($lang, 'deleteall_message_button'),
		     'undelete_message_button' => lang_text_utf8($lang, 'undelete_message_button'),
		     'not_spam' => lang_text_utf8($lang, 'not_spam'),
		     'header_score_name' => lang_text_utf8($lang, 'header_score_name'),
		     'header_from_name' => lang_text_utf8($lang, 'header_from_name'),
		     'header_subject_name' => lang_text_utf8($lang, 'header_subject_name'),
		     'header_date_name' => lang_text_utf8($lang, 'header_date_name'),
		     'content_message_type' => $message_type,
		     'content_message_hidden_page' => $page,
		     'buttons_blocked' => $buttons_blocked,
		     'buttons_deleted' => $buttons_deleted,
		     'content_sort_by' => $content_sort_by,
		     'sort_order' => $cgi_sort_order,
		     'sort_from' => $sort_from,
		     'sort_date' => $sort_date,
		     'sort_probability' => $sort_probability,
		     'sort_subject' => $sort_subject,
		     'order_ascending' => $order_ascending,
		     'order_descending' => $order_descending,
		     'message_page_info' => lang_text_utf8($lang, 'message_page_info', ($start + 1), ($end + 1), $mess_count, $page, $pages),
		     'page_jump' => lang_text_utf8($lang, 'page_jump'),
		     'content_message_page_previous' => $previous,
		     'content_message_page_next' => $next,
		     'content_message_page' => \@pgs,
		     'content_message_list' => \@messages,
                     'white_list' => $features{'list.white'}
		     );
		     
$r->add_part($content_tmpl);
$r->send;
